@@ -815,7 +815,7 @@ render(struct window *window, struct buffer *buffer)
 
 	glUniform1f(window->gl.offset_uniform, offset);
 	glUniformMatrix4fv(window->gl.reflection_uniform, 1, GL_FALSE,
-			   (GLfloat *) reflection.d);
+			   (GLfloat *) reflection.M.colmaj);
 
 	glClearColor(0.0,0.0, 0.0, 1.0);
 	glClear(GL_COLOR_BUFFER_BIT);
@@ -864,7 +864,7 @@ render_mandelbrot(struct window *window, struct buffer *buffer)
 
 	glUniform1f(window->gl.offset_uniform, offset);
 	glUniformMatrix4fv(window->gl.reflection_uniform, 1, GL_FALSE,
-			   (GLfloat *) reflection.d);
+			   (GLfloat *) reflection.M.colmaj);
 
 	glClearColor(0.6, 0.6, 0.6, 1.0);
 	glClear(GL_COLOR_BUFFER_BIT);
@@ -1110,6 +1110,12 @@ destroy_display(struct display *display)
 
 	free(display->modifiers);
 
+	if (display->direct_display)
+		weston_direct_display_v1_destroy(display->direct_display);
+
+	if (display->explicit_sync)
+		zwp_linux_explicit_synchronization_v1_destroy(display->explicit_sync);
+
 	if (display->dmabuf)
 		zwp_linux_dmabuf_v1_destroy(display->dmabuf);
 
