@@ -58,6 +58,7 @@ static char *option_term;
 static char *option_shell;
 
 static struct wl_list terminal_list;
+struct sigaction oldact;
 
 static struct terminal *
 terminal_create(struct display *display);
@@ -868,8 +869,8 @@ resize_handler(struct widget *widget,
 		terminal->pace_pipe = -1;
 	}
 	m = 2 * terminal->margin;
-	columns = (width - m) / (int32_t) terminal->average_width;
-	rows = (height - m) / (int32_t) terminal->extents.height;
+	columns = (int32_t) round((width - m) / terminal->average_width);
+	rows = (int32_t) round((height - m) / terminal->extents.height);
 
 	if (!window_is_fullscreen(terminal->window) &&
 	    !window_is_maximized(terminal->window)) {
@@ -1711,7 +1712,7 @@ handle_non_csi_escape(struct terminal *terminal, char code)
 		break;
 	case 'E':    /* NEL - Newline */
 		terminal->column = 0;
-		// fallthrough
+		FALLTHROUGH;
 	case 'D':    /* IND - Linefeed */
 		terminal->row += 1;
 		if (terminal->row > terminal->margin_bottom) {
@@ -1893,7 +1894,7 @@ handle_special_char(struct terminal *terminal, char c)
 		if (terminal->mode & MODE_LF_NEWLINE) {
 			terminal->column = 0;
 		}
-		/* fallthrough */
+		FALLTHROUGH;
 	case '\v':
 	case '\f':
 		terminal->row++;
@@ -3100,6 +3101,9 @@ terminal_run(struct terminal *terminal, const char *path)
 		close(pipes[0]);
 		setenv("TERM", option_term, 1);
 		setenv("COLORTERM", option_term, 1);
+
+		sigaction(SIGPIPE, &oldact, NULL);
+
 		if (execl(path, path, NULL)) {
 			printf("exec failed: %s\n", strerror(errno));
 			exit(EXIT_FAILURE);
@@ -3177,8 +3181,10 @@ int main(int argc, char *argv[])
 	 * socket whose reading end has been closed */
 	sigpipe.sa_handler = SIG_IGN;
 	sigemptyset(&sigpipe.sa_mask);
+	sigemptyset(&oldact.sa_mask);
+
 	sigpipe.sa_flags = 0;
-	sigaction(SIGPIPE, &sigpipe, NULL);
+	sigaction(SIGPIPE, &sigpipe, &oldact);
 
 	d = display_create(&argc, argv);
 	if (d == NULL) {
