@@ -56,9 +56,11 @@
 #include <libweston/backend-pipewire.h>
 #include <libweston/linux-dmabuf.h>
 #include <libweston/weston-log.h>
+#include "output-capture.h"
 #include "pixel-formats.h"
 #include "pixman-renderer.h"
 #include "renderer-gl/gl-renderer.h"
+#include "renderer-vulkan/vulkan-renderer.h"
 #include "shared/weston-egl-ext.h"
 
 struct pipewire_backend {
@@ -101,7 +103,7 @@ struct pipewire_head {
 };
 
 struct pipewire_frame_data {
-	struct weston_renderbuffer *renderbuffer;
+	weston_renderbuffer_t renderbuffer;
 	struct pipewire_memfd *memfd;
 	struct pipewire_dmabuf *dmabuf;
 };
@@ -266,14 +268,16 @@ pipewire_output_connect(struct pipewire_output *output)
 		/* TODO: Add support for modifier discovery and negotiation. */
 		uint64_t modifier[] = { DRM_FORMAT_MOD_LINEAR };
 		params[i++] = spa_pod_build_format(&builder,
-						   output->base.width, output->base.height,
+						   output->base.current_mode->width,
+						   output->base.current_mode->height,
 						   output->base.current_mode->refresh / 1000,
 						   output->pixel_format->format,
 						   modifier);
 	}
 
 	params[i++] = spa_pod_build_format(&builder,
-					   output->base.width, output->base.height,
+					   output->base.current_mode->width,
+					   output->base.current_mode->height,
 					   output->base.current_mode->refresh / 1000,
 					   output->pixel_format->format, NULL);
 
@@ -307,8 +311,8 @@ pipewire_output_enable_pixman(struct pipewire_output *output)
 	const struct pixman_renderer_output_options options = {
 		.use_shadow = true,
 		.fb_size = {
-			.width = output->base.width,
-			.height = output->base.height,
+			.width = output->base.current_mode->width,
+			.height = output->base.current_mode->height,
 		},
 		.format = output->pixel_format,
 	};
@@ -347,6 +351,29 @@ pipewire_output_enable_gl(struct pipewire_output *output)
 	return renderer->gl->output_fbo_create(&output->base, &options);
 }
 
+static int
+pipewire_output_enable_vulkan(struct pipewire_output *output)
+{
+	struct pipewire_backend *b = output->backend;
+	struct weston_renderer *renderer = b->compositor->renderer;
+	const struct weston_size fb_size = {
+		output->base.current_mode->width,
+		output->base.current_mode->height
+	};
+	const struct weston_geometry area = {
+		.x = 0,
+		.y = 0,
+		.width = fb_size.width,
+		.height = fb_size.height
+	};
+	const struct vulkan_renderer_surfaceless_options options = {
+		.fb_size = fb_size,
+		.area = area,
+	};
+
+	return renderer->vulkan->output_surfaceless_create(&output->base, &options);
+}
+
 static void
 pipewire_output_disable_gl(struct pipewire_output *output)
 {
@@ -355,6 +382,14 @@ pipewire_output_disable_gl(struct pipewire_output *output)
 	renderer->gl->output_destroy(&output->base);
 }
 
+static void
+pipewire_output_disable_vulkan(struct pipewire_output *output)
+{
+	struct weston_renderer *renderer = output->base.compositor->renderer;
+
+	renderer->vulkan->output_destroy(&output->base);
+}
+
 static int
 pipewire_output_enable(struct weston_output *base)
 {
@@ -373,6 +408,9 @@ pipewire_output_enable(struct weston_output *base)
 	case WESTON_RENDERER_GL:
 		ret = pipewire_output_enable_gl(output);
 		break;
+	case WESTON_RENDERER_VULKAN:
+		ret = pipewire_output_enable_vulkan(output);
+		break;
 	default:
 		unreachable("Valid renderer should have been selected");
 	}
@@ -398,6 +436,9 @@ err:
 	case WESTON_RENDERER_GL:
 		pipewire_output_disable_gl(output);
 		break;
+	case WESTON_RENDERER_VULKAN:
+		pipewire_output_disable_vulkan(output);
+		break;
 	default:
 		unreachable("Valid renderer should have been selected");
 	}
@@ -426,6 +467,9 @@ pipewire_output_disable(struct weston_output *base)
 	case WESTON_RENDERER_GL:
 		pipewire_output_disable_gl(output);
 		break;
+	case WESTON_RENDERER_VULKAN:
+		pipewire_output_disable_vulkan(output);
+		break;
 	default:
 		unreachable("Valid renderer should have been selected");
 	}
@@ -490,8 +534,8 @@ pipewire_output_create_dmabuf(struct pipewire_output *output)
 	uint64_t modifier[] = { DRM_FORMAT_MOD_LINEAR };
 
 	format = output->pixel_format;
-	width = output->base.width;
-	height = output->base.height;
+	width = output->base.current_mode->width;
+	height = output->base.current_mode->height;
 
 	linux_dmabuf_memory = renderer->dmabuf_alloc(renderer, width, height,
 						     format->format,
@@ -593,59 +637,6 @@ pipewire_output_stream_param_changed(void *data, uint32_t id,
 	pw_stream_update_params(output->stream, params, 2);
 }
 
-static struct weston_renderbuffer *
-pipewire_output_stream_add_buffer_pixman(struct pipewire_output *output,
-					 struct pw_buffer *buffer)
-{
-	struct weston_compositor *ec = output->base.compositor;
-	const struct weston_renderer *renderer = ec->renderer;
-	struct spa_buffer *buf = buffer->buffer;
-	struct spa_data *d = buf->datas;
-	const struct pixel_format_info *format;
-	unsigned int width;
-	unsigned int height;
-	unsigned int stride;
-	void *ptr;
-
-	format = output->pixel_format;
-	width = output->base.width;
-	height = output->base.height;
-	stride = width * format->bpp / 8;
-	ptr = d[0].data;
-
-	return renderer->pixman->create_image_from_ptr(&output->base,
-						       format, width, height,
-						       ptr, stride);
-}
-
-static struct weston_renderbuffer *
-pipewire_output_stream_add_buffer_gl(struct pipewire_output *output,
-				     struct pw_buffer *buffer)
-{
-	struct weston_compositor *ec = output->base.compositor;
-	const struct weston_renderer *renderer = ec->renderer;
-	struct spa_buffer *buf = buffer->buffer;
-	struct spa_data *d = buf->datas;
-	const struct pixel_format_info *format;
-	unsigned int width;
-	unsigned int height;
-	void *ptr;
-	struct pipewire_frame_data *frame_data = buffer->user_data;
-	struct pipewire_dmabuf *dmabuf = frame_data->dmabuf;
-
-	if (dmabuf)
-		return renderer->create_renderbuffer_dmabuf(&output->base,
-							    dmabuf->linux_dmabuf_memory);
-
-	format = output->pixel_format;
-	width = output->base.width;
-	height = output->base.height;
-	ptr = d[0].data;
-
-	return renderer->gl->create_fbo(&output->base,
-					format, width, height, ptr);
-}
-
 struct pipewire_memfd {
 	int fd;
 	unsigned int size;
@@ -665,8 +656,8 @@ pipewire_output_create_memfd(struct pipewire_output *output)
 	memfd = xzalloc(sizeof *memfd);
 
 	format = output->pixel_format;
-	width = output->base.width;
-	height = output->base.height;
+	width = output->base.current_mode->width;
+	height = output->base.current_mode->height;
 	stride = width * format->bpp / 8;
 	size = height * stride;
 
@@ -755,8 +746,15 @@ pipewire_output_stream_add_buffer(void *data, struct pw_buffer *buffer)
 			return;
 		}
 		pipewire_output_setup_dmabuf(output, buffer, dmabuf);
+
+		frame_data->renderbuffer =
+			renderer->create_renderbuffer_dmabuf(&output->base,
+							     dmabuf->linux_dmabuf_memory,
+							     NULL, NULL);
 		frame_data->dmabuf = dmabuf;
 	} else if (buffertype & (1u << SPA_DATA_MemFd)) {
+		const struct pixel_format_info *format = output->pixel_format;
+		int stride = output->base.current_mode->width * format->bpp / 8;
 		struct pipewire_memfd *memfd;
 
 		memfd = pipewire_output_create_memfd(output);
@@ -766,18 +764,12 @@ pipewire_output_stream_add_buffer(void *data, struct pw_buffer *buffer)
 			return;
 		}
 		pipewire_output_setup_memfd(output, buffer, memfd);
-		frame_data->memfd = memfd;
-	}
 
-	switch (renderer->type) {
-	case WESTON_RENDERER_PIXMAN:
-		frame_data->renderbuffer = pipewire_output_stream_add_buffer_pixman(output, buffer);
-		break;
-	case WESTON_RENDERER_GL:
-		frame_data->renderbuffer = pipewire_output_stream_add_buffer_gl(output, buffer);
-		break;
-	default:
-		unreachable("Valid renderer should have been selected");
+		frame_data->renderbuffer =
+			renderer->create_renderbuffer(&output->base, format,
+						      d[0].data, stride, NULL,
+						      NULL);
+		frame_data->memfd = memfd;
 	}
 }
 
@@ -800,21 +792,20 @@ pipewire_output_stream_remove_buffer(void *data, struct pw_buffer *buffer)
 
 	pipewire_output_debug(output, "remove buffer: %p", buffer);
 
-	if (frame_data->dmabuf) {
-		struct weston_compositor *ec = output->base.compositor;
-		const struct weston_renderer *renderer = ec->renderer;
-
-		renderer->remove_renderbuffer_dmabuf(&output->base,
-						     frame_data->renderbuffer);
+	if (frame_data->dmabuf)
 		pipewire_destroy_dmabuf(output, frame_data->dmabuf);
-	}
+
 	if (frame_data->memfd) {
 		munmap(d[0].data, d[0].maxsize);
 		pipewire_destroy_memfd(output, frame_data->memfd);
 	}
 
-	if (frame_data->renderbuffer)
-		weston_renderbuffer_unref(frame_data->renderbuffer);
+	if (frame_data->renderbuffer) {
+		struct weston_compositor *ec = output->base.compositor;
+		const struct weston_renderer *renderer = ec->renderer;
+
+		renderer->destroy_renderbuffer(frame_data->renderbuffer);
+	}
 	wl_list_for_each(fence_data, &output->fence_list, link) {
 		if (fence_data->buffer == buffer)
 			fence_data->buffer = NULL;
@@ -955,7 +946,7 @@ pipewire_submit_buffer(struct pipewire_output *output,
 	if (dmabuf)
 		stride = dmabuf->linux_dmabuf_memory->attributes->stride[0];
 	else
-		stride = output->base.width * pixel_format->bpp / 8;
+		stride = output->base.current_mode->width * pixel_format->bpp / 8;
 	size = output->base.height * stride;
 
 	spa_buffer = buffer->buffer;
@@ -1008,9 +999,20 @@ pipewire_schedule_submit_buffer(struct pipewire_output *output,
 	struct wl_event_loop *loop;
 	int fence_sync_fd;
 
-	fence_sync_fd = renderer->gl->create_fence_fd(&output->base);
-	if (fence_sync_fd == -1)
-		return -1;
+	switch (renderer->type) {
+	case WESTON_RENDERER_GL:
+		fence_sync_fd = renderer->gl->create_fence_fd(&output->base);
+		if (fence_sync_fd == -1)
+			return -1;
+		break;
+	case WESTON_RENDERER_VULKAN:
+		fence_sync_fd = renderer->vulkan->create_fence_fd(&output->base);
+		if (fence_sync_fd == -1)
+			return -1;
+		break;
+	default:
+		unreachable("invalid renderer");
+	}
 
 	fence_data = zalloc(sizeof *fence_data);
 	if (!fence_data) {
@@ -1042,6 +1044,7 @@ pipewire_output_repaint(struct weston_output *base)
 	struct pipewire_frame_data *frame_data;
 	pixman_region32_t damage;
 	bool submit_scheduled = false;
+	bool rendered = false;
 
 	assert(output);
 
@@ -1052,7 +1055,8 @@ pipewire_output_repaint(struct weston_output *base)
 
 	weston_output_flush_damage_for_primary_plane(base, &damage);
 
-	if (!pixman_region32_not_empty(&damage))
+	if (!pixman_region32_not_empty(&damage) &&
+	    !weston_output_has_renderer_capture_tasks(base))
 		goto out;
 
 	buffer = pw_stream_dequeue_buffer(output->stream);
@@ -1063,9 +1067,10 @@ pipewire_output_repaint(struct weston_output *base)
 	pipewire_output_debug(output, "dequeued buffer: %p", buffer);
 
 	frame_data = buffer->user_data;
-	if (frame_data->renderbuffer)
+	if (frame_data->renderbuffer) {
 		ec->renderer->repaint_output(&output->base, &damage, frame_data->renderbuffer);
-	else
+		rendered = true;
+	} else
 		output->base.full_repaint_needed = true;
 
 	if (buffer->buffer->datas[0].type == SPA_DATA_DmaBuf) {
@@ -1076,6 +1081,21 @@ pipewire_output_repaint(struct weston_output *base)
 		pipewire_submit_buffer(output, buffer);
 
 out:
+	/* We didn't have anywhere to render to, so capture tasks could be
+	 * left dangling. Shoot them down.
+	 */
+	if (!rendered && weston_output_has_renderer_capture_tasks(base)) {
+		int width = base->current_mode->width;
+		int height = base->current_mode->height;
+		struct weston_capture_task *ct;
+
+		while ((ct = weston_output_pull_capture_task(base,
+							     WESTON_OUTPUT_CAPTURE_SOURCE_FRAMEBUFFER,
+							     width, height,
+							     ec->read_format,
+							     NULL)))
+			weston_capture_task_retire_failed(ct, "No pipewire buffer");
+	}
 
 	pixman_region32_fini(&damage);
 
@@ -1116,6 +1136,19 @@ pipewire_ensure_matching_mode(struct weston_output *output, struct weston_mode *
 					target->refresh);
 }
 
+static void
+pipewire_set_dpms(struct weston_output *base, enum dpms_enum level)
+{
+	struct pipewire_output *output = to_pipewire_output(base);
+
+	if (level == WESTON_DPMS_ON)
+		weston_output_schedule_repaint(base);
+	else if (output->base.repaint_status == REPAINT_AWAITING_COMPLETION) {
+		wl_event_source_timer_update(output->finish_frame_timer, 0);
+		weston_output_schedule_repaint_reset(base);
+	}
+}
+
 static int
 pipewire_switch_mode(struct weston_output *base, struct weston_mode *target_mode)
 {
@@ -1142,14 +1175,13 @@ pipewire_switch_mode(struct weston_output *base, struct weston_mode *target_mode
 }
 
 static int
-pipewire_output_set_size(struct weston_output *base, int width, int height)
+pipewire_output_set_size(struct weston_output *base, int width, int height, int framerate)
 {
 	struct pipewire_output *output = to_pipewire_output(base);
 	struct weston_head *head;
 	struct pipewire_head *pw_head;
 	struct weston_mode *current_mode;
 	struct weston_mode init_mode;
-	int framerate = -1;
 
 	/* We can only be called once. */
 	assert(!output->base.current_mode);
@@ -1161,7 +1193,8 @@ pipewire_output_set_size(struct weston_output *base, int width, int height)
 			width = pw_head->config.width;
 		if (height == -1)
 			height = pw_head->config.height;
-		framerate = pw_head->config.framerate;
+		if (framerate == -1)
+			framerate = pw_head->config.framerate;
 	}
 	if (framerate == -1 || width == -1 || height == -1)
 		return -1;
@@ -1179,7 +1212,7 @@ pipewire_output_set_size(struct weston_output *base, int width, int height)
 	output->base.repaint = pipewire_output_repaint;
 	output->base.assign_planes = NULL;
 	output->base.set_backlight = NULL;
-	output->base.set_dpms = NULL;
+	output->base.set_dpms = pipewire_set_dpms;
 	output->base.switch_mode = pipewire_switch_mode;
 
 	return 0;
@@ -1352,6 +1385,16 @@ pipewire_backend_create(struct weston_compositor *compositor,
 							      &options.base);
 			break;
 		}
+		case WESTON_RENDERER_VULKAN: {
+			const struct vulkan_renderer_display_options options = {
+				.formats = backend->formats,
+				.formats_count = backend->formats_count,
+			};
+			ret = weston_compositor_init_renderer(compositor,
+							      WESTON_RENDERER_VULKAN,
+							      &options.base);
+			break;
+		}
 		default:
 			weston_log("Unsupported renderer requested\n");
 			goto err_compositor;
@@ -1417,6 +1460,7 @@ weston_backend_init(struct weston_compositor *compositor,
 		switch (compositor->renderer->type) {
 		case WESTON_RENDERER_PIXMAN:
 		case WESTON_RENDERER_GL:
+		case WESTON_RENDERER_VULKAN:
 			break;
 		default:
 			weston_log("Renderer not supported by PipeWire backend\n");
