@@ -25,26 +25,24 @@
 
 #include "config.h"
 
-#include "compositor.h"
+#include <libweston/libweston.h>
 
 struct weston_launcher;
 
 struct launcher_interface {
+	char *name;
 	int (* connect) (struct weston_launcher **launcher_out, struct weston_compositor *compositor,
-			 int tty, const char *seat_id, bool sync_drm);
+			 const char *seat_id, bool sync_drm);
 	void (* destroy) (struct weston_launcher *launcher);
 	int (* open) (struct weston_launcher *launcher, const char *path, int flags);
 	void (* close) (struct weston_launcher *launcher, int fd);
 	int (* activate_vt) (struct weston_launcher *launcher, int vt);
-	void (* restore) (struct weston_launcher *launcher);
 	/* Get the number of the VT weston is running in */
 	int (* get_vt) (struct weston_launcher *launcher);
 };
 
 struct weston_launcher {
-	struct launcher_interface *iface;
+	const struct launcher_interface *iface;
 };
 
-extern struct launcher_interface launcher_logind_iface;
-extern struct launcher_interface launcher_weston_launch_iface;
-extern struct launcher_interface launcher_direct_iface;
+extern const struct launcher_interface launcher_libseat_iface;
