@@ -29,6 +29,7 @@
 #include "backend.h"
 #include "libweston-internal.h"
 #include "shared/timespec-util.h"
+#include "shared/string-helpers.h"
 #include <libweston/backend-drm.h>
 #include <libweston/weston-log.h>
 
@@ -66,7 +67,6 @@ struct weston_pipewire {
 
 struct pipewire_output {
 	struct weston_output *output;
-	void (*saved_destroy)(struct weston_output *output);
 	int (*saved_enable)(struct weston_output *output);
 	int (*saved_disable)(struct weston_output *output);
 	int (*saved_start_repaint_loop)(struct weston_output *output);
@@ -149,6 +149,16 @@ lookup_pipewire_output(struct weston_output *base_output)
 	struct weston_pipewire *pipewire = weston_pipewire_get(c);
 	struct pipewire_output *output;
 
+	/* XXX: This could happen on the compositor shutdown path with our
+	 * destroy listener being removed, and pipewire_output_destroy() being
+	 * called as a virtual destructor.
+	 *
+	 * See https://gitlab.freedesktop.org/wayland/weston/-/issues/591 for
+	 * an alternative to the shutdown sequence.
+	 */
+	if (!pipewire)
+		return NULL;
+
 	wl_list_for_each(output, &pipewire->output_list, link) {
 		if (output->output == base_output)
 			return output;
@@ -182,7 +192,10 @@ pipewire_output_handle_frame(struct pipewire_output *output, int fd,
 
 	if ((h = spa_buffer_find_meta_data(spa_buffer, SPA_META_Header,
 				     sizeof(struct spa_meta_header)))) {
-		h->pts = -1;
+		struct timespec ts;
+
+		clock_gettime(CLOCK_MONOTONIC, &ts);
+		h->pts = SPA_TIMESPEC_TO_NSEC(&ts);
 		h->flags = 0;
 		h->seq = output->seq++;
 		h->dts_offset = 0;
@@ -310,17 +323,19 @@ pipewire_output_destroy(struct weston_output *base_output)
 	struct pipewire_output *output = lookup_pipewire_output(base_output);
 	struct weston_mode *mode, *next;
 
+	if (!output)
+		return;
+
+	weston_head_release(output->head);
+
 	wl_list_for_each_safe(mode, next, &base_output->mode_list, link) {
 		wl_list_remove(&mode->link);
 		free(mode);
 	}
 
-	output->saved_destroy(base_output);
-
 	pw_stream_destroy(output->stream);
 
 	wl_list_remove(&output->link);
-	weston_head_release(output->head);
 	free(output->head);
 	free(output);
 }
@@ -509,7 +524,6 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	const struct weston_drm_virtual_output_api *api;
 	const char *make = "Weston";
 	const char *model = "Virtual Display";
-	const char *serial_number = "unknown";
 	const char *connector_name = "pipewire";
 	char *remoting_name;
 
@@ -535,14 +549,12 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	pw_stream_add_listener(output->stream, &output->stream_listener,
 			       &stream_events, output);
 
-	output->output = api->create_output(c, name);
+	output->output = api->create_output(c, name, pipewire_output_destroy);
 	if (!output->output) {
 		weston_log("Cannot create virtual output\n");
 		goto err;
 	}
 
-	output->saved_destroy = output->output->destroy;
-	output->output->destroy = pipewire_output_destroy;
 	output->saved_enable = output->output->enable;
 	output->output->enable = pipewire_output_enable;
 	output->saved_disable = output->output->disable;
@@ -550,10 +562,10 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	output->pipewire = pipewire;
 	wl_list_insert(pipewire->output_list.prev, &output->link);
 
-	asprintf(&remoting_name, "%s-%s", connector_name, name);
+	str_printf(&remoting_name, "%s-%s", connector_name, name);
 	weston_head_init(head, remoting_name);
 	weston_head_set_subpixel(head, WL_OUTPUT_SUBPIXEL_NONE);
-	weston_head_set_monitor_strings(head, make, model, serial_number);
+	weston_head_set_monitor_strings(head, make, model, NULL);
 	head->compositor = c;
 	output->head = head;
 
@@ -634,13 +646,19 @@ weston_pipewire_destroy(struct wl_listener *l, void *data)
 {
 	struct weston_pipewire *pipewire =
 		wl_container_of(l, pipewire, destroy_listener);
+	struct pipewire_output *p_output, *p_output_next;
 
 	weston_log_scope_destroy(pipewire->debug);
 	pipewire->debug = NULL;
 
+	wl_list_for_each_safe(p_output, p_output_next, &pipewire->output_list, link)
+		pipewire_output_destroy(p_output->output);
+
 	wl_event_source_remove(pipewire->loop_source);
 	pw_loop_leave(pipewire->loop);
 	pw_loop_destroy(pipewire->loop);
+
+	free(pipewire);
 }
 
 static struct weston_pipewire *
