@@ -35,6 +35,7 @@
 #include "shared/os-compatibility.h"
 #include "weston-test-client-helper.h"
 #include "weston-test-fixture-compositor.h"
+#include "weston-test-assert.h"
 
 static enum test_result_code
 fixture_setup(struct weston_test_harness *harness)
@@ -151,7 +152,7 @@ create_bad_shm_buffer(struct client *client, int width, int height)
 	int fd;
 
 	fd = create_anonymous_file_without_seals(size);
-	assert(fd >= 0);
+	test_assert_int_ge(fd, 0);
 
 	pool = wl_shm_create_pool(shm, fd, size);
 	buffer = wl_shm_pool_create_buffer(pool, 0, width, height, stride,
@@ -161,7 +162,7 @@ create_bad_shm_buffer(struct client *client, int width, int height)
 	/* Truncate the file to a small size, so that the compositor
 	 * will access it out-of-bounds, and hit SIGBUS.
 	 */
-	assert(ftruncate(fd, 12) == 0);
+	test_assert_int_eq(ftruncate(fd, 12), 0);
 	close(fd);
 
 	return buffer;
@@ -176,7 +177,7 @@ TEST(test_truncated_shm_file)
 	int frame;
 
 	client = create_client_and_test_surface(46, 76, 111, 134);
-	assert(client);
+	test_assert_ptr_not_null(client);
 	surface = client->surface->wl_surface;
 
 	bad_buffer = create_bad_shm_buffer(client, 200, 200);
@@ -193,4 +194,6 @@ TEST(test_truncated_shm_file)
 
 	wl_buffer_destroy(bad_buffer);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
