@@ -32,6 +32,7 @@
 #include "weston-test-client-helper.h"
 #include "wayland-server-protocol.h"
 #include "weston-test-fixture-compositor.h"
+#include "weston-test-assert.h"
 
 static enum test_result_code
 fixture_setup(struct weston_test_harness *harness)
@@ -65,18 +66,18 @@ get_linux_explicit_synchronization(struct client *client)
 			continue;
 
 		if (global_sync)
-			assert(!"Multiple linux explicit sync objects");
+			test_assert_not_reached("Multiple linux explicit sync objects");
 
 		global_sync = g;
 	}
 
-	assert(global_sync);
-	assert(global_sync->version == 2);
+	test_assert_ptr_not_null(global_sync);
+	test_assert_u32_eq(global_sync->version, 2);
 
 	sync = wl_registry_bind(
 			client->wl_registry, global_sync->name,
 			&zwp_linux_explicit_synchronization_v1_interface, 2);
-	assert(sync);
+	test_assert_ptr_not_null(sync);
 
 	return sync;
 }
@@ -85,7 +86,7 @@ static struct client *
 create_test_client(void)
 {
 	struct client *cl = create_client_and_test_surface(0, 0, 100, 100);
-	assert(cl);
+	test_assert_ptr_not_null(cl);
 	return cl;
 }
 
@@ -115,6 +116,8 @@ TEST(second_surface_synchronization_on_surface_raises_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync1);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_acquire_fence_with_invalid_fence_raises_error)
@@ -127,7 +130,7 @@ TEST(set_acquire_fence_with_invalid_fence_raises_error)
 			sync, client->surface->wl_surface);
 	int pipefd[2] = { -1, -1 };
 
-	assert(pipe(pipefd) == 0);
+	test_assert_int_eq(pipe(pipefd), 0);
 
 	zwp_linux_surface_synchronization_v1_set_acquire_fence(surface_sync,
 							       pipefd[0]);
@@ -141,6 +144,8 @@ TEST(set_acquire_fence_with_invalid_fence_raises_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(set_acquire_fence_on_destroyed_surface_raises_error)
@@ -153,7 +158,7 @@ TEST(set_acquire_fence_on_destroyed_surface_raises_error)
 			sync, client->surface->wl_surface);
 	int pipefd[2] = { -1, -1 };
 
-	assert(pipe(pipefd) == 0);
+	test_assert_int_eq(pipe(pipefd), 0);
 
 	wl_surface_destroy(client->surface->wl_surface);
 	client->surface->wl_surface = NULL;
@@ -169,6 +174,8 @@ TEST(set_acquire_fence_on_destroyed_surface_raises_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(second_buffer_release_in_commit_raises_error)
@@ -199,6 +206,8 @@ TEST(second_buffer_release_in_commit_raises_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(get_release_without_buffer_raises_commit_error)
@@ -224,6 +233,8 @@ TEST(get_release_without_buffer_raises_commit_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(get_release_on_destroyed_surface_raises_error)
@@ -249,6 +260,8 @@ TEST(get_release_on_destroyed_surface_raises_error)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(get_release_after_commit_succeeds)
@@ -260,10 +273,13 @@ TEST(get_release_after_commit_succeeds)
 	struct zwp_linux_surface_synchronization_v1 *surface_sync =
 		zwp_linux_explicit_synchronization_v1_get_synchronization(
 			sync, surface);
-	struct buffer *buf1 = create_shm_buffer_a8r8g8b8(client, 100, 100);
+	struct buffer *buf1;
+	pixman_color_t black;
 	struct zwp_linux_buffer_release_v1 *buffer_release1;
 	struct zwp_linux_buffer_release_v1 *buffer_release2;
 
+	color_rgb888(&black, 0, 0, 0);
+	buf1 = create_shm_buffer_solid(client, 100, 100, &black);
 	buffer_release1 =
 		zwp_linux_surface_synchronization_v1_get_release(surface_sync);
 	client_roundtrip(client);
@@ -281,6 +297,8 @@ TEST(get_release_after_commit_succeeds)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 static void
@@ -288,7 +306,7 @@ buffer_release_fenced_handler(void *data,
 			      struct zwp_linux_buffer_release_v1 *buffer_release,
 			      int32_t fence)
 {
-	assert(!"Fenced release not supported yet");
+	test_assert_not_reached("Fenced release not supported yet");
 }
 
 static void
@@ -320,8 +338,9 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	struct zwp_linux_surface_synchronization_v1 *surface_sync =
 		zwp_linux_explicit_synchronization_v1_get_synchronization(
 			sync, client->surface->wl_surface);
-	struct buffer *buf1 = create_shm_buffer_a8r8g8b8(client, 100, 100);
-	struct buffer *buf2 = create_shm_buffer_a8r8g8b8(client, 100, 100);
+	struct buffer *buf1;
+	struct buffer *buf2;
+	pixman_color_t black;
 	struct wl_surface *surface = client->surface->wl_surface;
 	struct zwp_linux_buffer_release_v1 *buffer_release1;
 	struct zwp_linux_buffer_release_v1 *buffer_release2;
@@ -329,6 +348,10 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	int buf_released2 = 0;
 	int frame;
 
+	color_rgb888(&black, 0, 0, 0);
+	buf1 = create_shm_buffer_solid(client, 100, 100, &black);
+	buf2 = create_shm_buffer_solid(client, 100, 100, &black);
+
 	buffer_release1 =
 		zwp_linux_surface_synchronization_v1_get_release(surface_sync);
 	zwp_linux_buffer_release_v1_add_listener(buffer_release1,
@@ -340,7 +363,7 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	frame_callback_wait(client, &frame);
 	/* No release event should have been emitted yet (we are using the
 	 * pixman renderer, which holds buffers until they are replaced). */
-	assert(buf_released1 == 0);
+	test_assert_int_eq(buf_released1, 0);
 
 	buffer_release2 =
 		zwp_linux_surface_synchronization_v1_get_release(surface_sync);
@@ -353,8 +376,8 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	frame_callback_wait(client, &frame);
 	/* Check that exactly one buffer_release event was emitted for the
 	 * previous commit (buf1). */
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 0);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 0);
 
 	wl_surface_attach(surface, buf1->proxy, 0, 0);
 	frame_callback_set(surface, &frame);
@@ -362,8 +385,8 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	frame_callback_wait(client, &frame);
 	/* Check that exactly one buffer_release event was emitted for the
 	 * previous commit (buf2). */
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 1);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 1);
 
 	buffer_destroy(buf2);
 	buffer_destroy(buf1);
@@ -372,6 +395,8 @@ TEST(get_release_events_are_emitted_for_different_buffers)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
@@ -382,7 +407,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	struct zwp_linux_surface_synchronization_v1 *surface_sync =
 		zwp_linux_explicit_synchronization_v1_get_synchronization(
 			sync, client->surface->wl_surface);
-	struct buffer *buf = create_shm_buffer_a8r8g8b8(client, 100, 100);
+	struct buffer *buf;
+	pixman_color_t black;
 	struct wl_surface *surface = client->surface->wl_surface;
 	struct zwp_linux_buffer_release_v1 *buffer_release1;
 	struct zwp_linux_buffer_release_v1 *buffer_release2;
@@ -390,6 +416,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	int buf_released2 = 0;
 	int frame;
 
+	color_rgb888(&black, 0, 0, 0);
+	buf = create_shm_buffer_solid(client, 100, 100, &black);
 	buffer_release1 =
 		zwp_linux_surface_synchronization_v1_get_release(surface_sync);
 	zwp_linux_buffer_release_v1_add_listener(buffer_release1,
@@ -401,7 +429,7 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	frame_callback_wait(client, &frame);
 	/* No release event should have been emitted yet (we are using the
 	 * pixman renderer, which holds buffers until they are replaced). */
-	assert(buf_released1 == 0);
+	test_assert_int_eq(buf_released1, 0);
 
 	buffer_release2 =
 		zwp_linux_surface_synchronization_v1_get_release(surface_sync);
@@ -414,8 +442,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	frame_callback_wait(client, &frame);
 	/* Check that exactly one buffer_release event was emitted for the
 	 * previous commit (buf). */
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 0);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 0);
 
 	wl_surface_attach(surface, buf->proxy, 0, 0);
 	frame_callback_set(surface, &frame);
@@ -423,8 +451,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	frame_callback_wait(client, &frame);
 	/* Check that exactly one buffer_release event was emitted for the
 	 * previous commit (buf again). */
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 1);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 1);
 
 	buffer_destroy(buf);
 	zwp_linux_buffer_release_v1_destroy(buffer_release2);
@@ -432,6 +460,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_surface)
 	zwp_linux_surface_synchronization_v1_destroy(surface_sync);
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
 
 TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
@@ -448,14 +478,19 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
 	struct zwp_linux_surface_synchronization_v1 *surface_sync2 =
 		zwp_linux_explicit_synchronization_v1_get_synchronization(
 			sync, surface2);
-	struct buffer *buf1 = create_shm_buffer_a8r8g8b8(client, 100, 100);
-	struct buffer *buf2 = create_shm_buffer_a8r8g8b8(client, 100, 100);
+	struct buffer *buf1;
+	struct buffer *buf2;
+	pixman_color_t black;
 	struct zwp_linux_buffer_release_v1 *buffer_release1;
 	struct zwp_linux_buffer_release_v1 *buffer_release2;
 	int buf_released1 = 0;
 	int buf_released2 = 0;
 	int frame;
 
+	color_rgb888(&black, 0, 0, 0);
+	buf1 = create_shm_buffer_solid(client, 100, 100, &black);
+	buf2 = create_shm_buffer_solid(client, 100, 100, &black);
+
 	weston_test_move_surface(client->test->weston_test, surface2, 0, 0);
 
 	/* Attach buf1 to both surface1 and surface2. */
@@ -479,8 +514,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
 	wl_surface_commit(surface2);
 	frame_callback_wait(client, &frame);
 
-	assert(buf_released1 == 0);
-	assert(buf_released2 == 0);
+	test_assert_int_eq(buf_released1, 0);
+	test_assert_int_eq(buf_released2, 0);
 
 	/* Attach buf2 to surface1, and check that a buffer_release event for
 	 * the previous commit (buf1) for that surface is emitted. */
@@ -489,8 +524,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
 	wl_surface_commit(surface1);
 	frame_callback_wait(client, &frame);
 
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 0);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 0);
 
 	/* Attach buf2 to surface2, and check that a buffer_release event for
 	 * the previous commit (buf1) for that surface is emitted. */
@@ -499,8 +534,8 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
 	wl_surface_commit(surface2);
 	frame_callback_wait(client, &frame);
 
-	assert(buf_released1 == 1);
-	assert(buf_released2 == 1);
+	test_assert_int_eq(buf_released1, 1);
+	test_assert_int_eq(buf_released2, 1);
 
 	buffer_destroy(buf2);
 	buffer_destroy(buf1);
@@ -511,4 +546,6 @@ TEST(get_release_events_are_emitted_for_same_buffer_on_different_surfaces)
 	zwp_linux_explicit_synchronization_v1_destroy(sync);
 	surface_destroy(other_surface);
 	client_destroy(client);
+
+	return RESULT_OK;
 }
